package ot_execute;
use strict;
#use FindBin;					# if it works: use FinBin
#use lib $FindBin::Bin;				# instead of static path
use lib '/data/httpd/opentheory/otperl';	# or: adjust path

use ot_const qw( %ot %r %f &is &is_err %opt );
use ot_maillib qw( &debug &mailerror &usermail &split_email_addr &split_addr );
use ot_dblib qw( &db_connect &db_disconnect &db_insert &db_delete
&db_select_col &db_select_row &db_select_val &db_cursor_open &db_cursor_get
&db_cursor_close &db_update_col &db_insert_member &db_get_msg &db_get_mail_msg
&db_set_status &db_get_memb_name_email &db_get_projabb &db_get_pidlist &db_is_project );
use ot_activity qw( &calc_activity );
use vars qw(@ISA @EXPORT_OK $VERSION);
use Exporter;
$VERSION = 1.04;
@ISA = qw(Exporter);
@EXPORT_OK = qw( &execute );

=head1 NAME

ot_execute - executes all commands stored in table command

=head1 SYNOPSIS

Scans table 'command' and executes all commands.

=head1 DESCRIPTION

All functions and procedures are described below.

=head2 initmsg - initializes a msg hash, see &sendmsg

S<in-param:	variable parameter list as key-value-pairs
returns:	%msg - msg hash>

The msg hash is used to obtain a message from database. This message
might contain variables, which will be interpreted in given context.
Valid variables in the context are (not all of them are initialized
with every command!):
S<
+ $cmd: command
+ $proj: short project name
+ $projlow: $proj in lower cases
+ $fn: user's first name (only if mid exists)
+ $ln: user's last name (only if mid exists)
+ $email: user's email address (only if mid exists)
+ $pw: user's password (only if mid exists)
+ $long: user's long format email address (incl. alias)
+ $cset: char of text mail send by user
+ $subj: subject of mail send by user
+ $body: body of mail send by or to user
+ $domain: mail domain
+ $fqdomain: fully qualified domain
+ $text: multi purpose text>

=cut

sub initmsg {
	my %msg = (
		mid	=> 0,	# member id
		proj	=> '',	# short project name
		from	=> '',	# sender (if empty: listmaster)
		long	=> '',	# long address
		addr	=> '',	# comma separated short address list
		cset	=> '',	# charset of user mail
		subj	=> '',	# subject of user mail
		rply	=> '',	# reply-to string
		cmd	=> '',	# command name
		text	=> '',	# multi purpose text
		body	=> '',	# body of user mail or for user mail
		@_,		# list of variable key-value-pairs
	);			# overwriting initial values
	return( %msg );
}

=head2 sendmsg - sends a user mail, msg is read from db

S<in-param:	$dbh - database handle
in-param:	$msg_id - message id, used to read msg from db
in-param:	$msgvar - variables for msg to interpret
returns:	$status - processing status>

=cut

sub sendmsg {
	my( $dbh, $msg_id, %msgvar ) = @_;
	my $mid = defined ($msgvar{mid}) ? $msgvar{mid} : '';
	my $proj = defined ($msgvar{proj}) ? $msgvar{proj} : '';
	my $from = defined ($msgvar{from}) ? $msgvar{from} : '';
	my $long = defined ($msgvar{long}) ? $msgvar{long} : '';
	my $addr = defined ($msgvar{addr}) ? $msgvar{addr} : '';
	my $cset = defined ($msgvar{cset}) ? $msgvar{cset} : '';
	my $subj = defined ($msgvar{subj}) ? $msgvar{subj} : '';
	my $rply = defined ($msgvar{rply}) ? $msgvar{rply} : '';
	my $cmd = defined ($msgvar{cmd}) ? $msgvar{cmd} : '';
	my $text = defined ($msgvar{text}) ? $msgvar{text} : '';
	my $body = defined ($msgvar{body}) ? $msgvar{body} : '';
	my $domain = $ot{maildom};
	my $fqdomain = $ot{fqdomain};
	my $projlow = $proj;
	$projlow =~ tr/[A-Z]/[a-z]/; 
	my( $fn, $ln, $email, $msg, $alias ) = ( '', '', '', '', '' );

	if( $mid ){			# read user context if mid is not 0
		my %rec = db_select_row( $dbh, 'member',
			'first_n, last_n, email',				# fields
			"memb_id = $mid"					# where
		);
		if( !exists( $rec{email} )) { mailerror( 'Fatal error in ot_execute.send' ); }
		$fn = $rec{first_n};
		$ln = $rec{last_n};
		$email = $rec{email};
		$long = qq("$fn $ln" <$email>);
	} elsif( $addr ) {		# take address list if present
		$email = $addr;
	} else {			# otherwise use long address
		( $email, $alias ) = split_email_addr( $long );
		( $fn, $ln ) = split_addr( $alias, $email );
	}
	if( !( $_ = db_get_mail_msg( $dbh, $msg_id, 'de' ))) {
		mailerror( "ot_execute.sendmsg: missing msg" );
		return( $r{error} );
	}
	debug( "msg: $_", 1 );
	s/"/\\"/g;
	if( /(.+)%(.+)/ ){			# '%' is delimiter of subject%msg
		$subj = eval qq( "$1" );	# interpret in context!
		if( $@ ) { mailerror( "Error in eval 1: $@" ); }
		$_ = $2;
	}
	$msg = eval qq( "$_" );		# interpret in context!
	if( $@ ) { mailerror( "Error in eval 2: $@" ); }
	debug( "send - msg: $msg", 3 );
	usermail( $from, $long, $email, $subj, $rply, $cset, $msg );
	return( $r{ok_finished} );
}

=head2 which - sends a list with alls project of a member

S<in-param:	$dbh - database handle
in-param:	$mid _ member id
returns:	$status - processing status>

=cut

sub which {
	my( $dbh, $mid ) = @_;
	debug( "which: mid: $mid", 1 );
	my $status = $r{error};
	my %msg = db_get_msg( $dbh, 10, 'de' );
	my %val = ();
	my( $txt, $st )= ( '', '' );
	my $i = 0;
	my $cur = db_cursor_open( $dbh,
		'metatext m, projmemb p',						# tables
		'm.title, m.titlabb, m.vers_id, m.status, m.maillst, p.ismaint',	# cols
		"p.proj_id = m.proj_id and p.memb_id = $mid and m.status <> 'arc'	# where
		order by m.titlabb, m.vers_id"						# sort
	);
	my $res = 1;
	while( $res ) {
		%val = db_cursor_get( $cur );
		$res = exists( $val{titlabb} );
		if( $res ){
			if( $val{status} eq 'act' ) { $st = $msg{71}; }
			elsif( $val{status} eq 'ini' ) { $st = $msg{72}; }
			elsif( $val{status} eq 'fin' ) { $st = $msg{73}; }
			#$st =~ tr/[A-Z]/[a-z]/;
			$txt .= "- $msg{1} '$val{titlabb}'";
			if( $val{ismaint} ) { $txt .= " ($msg{4}!)"; }
			$txt .= ":\n  $val{title}\n";
			$txt .= "  $msg{74} $val{vers_id}, $msg{3}: $st\n";
			$txt .= "  $msg{2}: $val{maillst}\n\n";
			$i++;
		}
	}
	db_cursor_close( $cur );
	$txt .= "$i";
	my %msgvar = initmsg( mid => $mid, body => $txt );
	$status = sendmsg( $dbh, 106, %msgvar );
	return( $status );
}

=head2 who - sends a list with all members of a project

S<in-param:	$dbh - database handle
in-param:	$pid - project id
in-param:	$mid _ member id
returns:	$status - processing status>

=cut

sub who {
	my( $dbh, $pid, $mid ) = @_;
	debug( "who: pid: $pid, mid: $mid", 1 );
	my $status = $r{error};
	my %val = ();
	my $txt = '';
	my $i = 0;
	my $pjn = db_get_projabb( $dbh, $pid );
	my $cur = db_cursor_open( $dbh,
		'member m, projmemb p',				# tables
		'm.first_n, m.last_n, m.email',			# cols
		"p.memb_id = m.memb_id and p.proj_id = $pid"	# where
	);
	my $res = 1;
	while( $res ) {
		%val = db_cursor_get( $cur );
		$res = exists( $val{email} );
		if( $res ){
			$txt .= "$val{first_n} $val{last_n} <$val{email}>\n";
			$i++;
		}
	}
	db_cursor_close( $cur );
	$txt .= "\n$i";
	my %msgvar = initmsg( mid => $mid, proj => $pjn, body => $txt );
	$status = sendmsg( $dbh, 107, %msgvar );
	return( $status );
}

=head2 subscribe - subscribes a member and adds a project

S<in-param:	$dbh - database handle
in-param:	$pid - project id
in-param:	$mid - member Id (0, if not a ot-member)
in-param:	$arg - argument (values)
returns:	$status - processing status>

=cut

sub subscribe {
	my( $dbh, $pid, $mid, $arg ) = @_;
	debug( "subscribe: pid: $pid, mid: $mid, arg: $arg", 1 );
	my $status = $r{error};
	my $whr;
	my $val;
	my %rec;
	my %msgvar;
	if( !$mid ) {
		my( $addr, $alias ) = split_email_addr( $arg );
		my( $fn, $ln ) = split_addr( $alias, $addr );
		my $pw = int( rand( 1000000 ));		# not encrypted!
		%rec = (
			last_n	=> $ln,
			first_n	=> $fn,
			email	=> $addr,
			passwd	=> $pw,
			opt	=> 21
		);
		$mid = db_insert_member( $dbh, %rec );
		if( is_err( $mid )) { return( $status ); }
		# encrypt pw
		$whr = "memb_id = $mid";
		$val = 'password(passwd), subscr = now()';
		if( db_update_col( $dbh, 'member', 'passwd', $val, $whr )) {
			%msgvar = initmsg( mid => $mid, text => $pw );
			$status = sendmsg( $dbh, 102, %msgvar );
		}
	}
	if( !is_err( $mid ) && ( $pid > 0 )) {
		my $req = 0;	# open project
		$whr = "proj_id = $pid and status = 'act'";
		if( $val = db_select_val( $dbh, 'metatext', 'pjtype', $whr )) {
			$req = ( $val & 8 ) == 8;	# closed project?
		}
		%rec = (
			proj_id	=> $pid,
			memb_id	=> $mid,
			request	=> $req
		);
		if( !db_insert( $dbh, 'projmemb', %rec )) {
			mailerror( 'Fatal error: ot_execute.subscribe.insert_projmemb' );
			return( $r{error} );
		}
		my $pjn = db_get_projabb( $dbh, $pid );
		if( $req ){						# closed project
			# select maintainer data
			$whr = "proj_id = $pid and status = 'act'";
			my $maint_id = db_select_val( $dbh, 'metatext', 'memb_id', $whr );
			# insert commit request to maintainer
			my $i = int( rand( 1000000000 ));
			%rec = (
				msg_id	=> "request$i",
				proj_addr => "commit*$pjn",
				proj_id	=> $pid,
				sender	=> "webmaster\@$ot{maildom}",
				memb_id	=> $maint_id
			);
			if( !db_insert( $dbh, 'receive', %rec )) {
				mailerror( 'Fatal error: ot_execute.subscribe.insert_receive' );
				return( $r{error} );
			}
			$whr = "msg_id = 'request$i'";
			$val = db_select_val( $dbh, 'receive', 'rec_id', $whr );
			%rec = (
				rec_id	=> $val,
				cmd_id	=> $i,
				cmdkey	=> 'commit',
				cmdval	=> $mid,
				stcode	=> 2,
				status	=> 'ok_requested'
			);
			if( !db_insert( $dbh, 'command', %rec )) {
				mailerror( 'Fatal error: ot_execute.subscribe.insert_command' );
				return( $r{error} );
			}
                        # send commit request to maintainer
                        my $reply = "confirm*commit.$i\@$ot{maildom}";
			$val = db_get_memb_name_email( $dbh, $mid );
			%msgvar = initmsg(
				mid	=> $maint_id,
				text	=> $val,
				rply	=> $reply,
				proj	=> $pjn
			);
			$status = sendmsg( $dbh, 110, %msgvar );
			# send msg to subscriber: commitment requested
			%msgvar = initmsg( mid => $mid, proj => $pjn );
			$status = sendmsg( $dbh, 111, %msgvar );
		} else {						# open project
			$whr = "proj_id = $pid and status = 'act'";
			$val = db_select_val( $dbh, 'metatext', 'maillst', $whr );
			%msgvar = initmsg(
				mid	=> $mid,
				text	=> $val,
				proj	=> $pjn
			);
			$status = sendmsg( $dbh, 104, %msgvar );
		}
	}
	return( $status );
}

=head2 unsubscribe - unsubscribes a member from a project

S<in-param:	$dbh - database handle
in-param:	$pid - project id
in-param:	$mid - member Id
returns:	$status - processing status>

=cut

sub unsubscribe {
	my( $dbh, $pid, $mid ) = @_;
	debug( "unsubscribe: pid: $pid, mid: $mid", 1 );
	my $status = $r{error};
	if( $pid < 0 ){
		my $long = db_get_memb_name_email( $dbh, $mid );
		if( db_delete( $dbh, 'member', "memb_id = $mid" )) {
			my %msgvar = initmsg( long => $long );
			$status = sendmsg( $dbh, 103, %msgvar );
		}
	}
	elsif( db_delete( $dbh, 'projmemb', "proj_id = $pid and memb_id = $mid" )) {
		my $pjn = db_get_projabb( $dbh, $pid );
		my %msgvar = initmsg( mid => $mid, proj => $pjn );
		$status = sendmsg( $dbh, 105, %msgvar );
	}
	return( $status );
}

=head2 commit - commit by maintainer on subscribe request 

S<in-param:	$dbh - database handle
in-param:	$pid - project id
in-param:	$maint_id - maintainer id - to commit subscr on closed project
in-param:	$memb_id - member id - requested subscription on closed project
returns:	$status - processing status>

=cut

sub commit {
	my( $dbh, $pid, $maint_id, $memb_id ) = @_;
	debug( "commit: pid: $pid, maint_id: $maint_id, memb_id: $memb_id", 1 );
	# maint_id is currently unused - does maint again need a mail confirmation?
	my $status = $r{error};
	my $qry = "proj_id = $pid and memb_id = $memb_id and request = 1";
	if( db_update_col( $dbh, 'projmemb', 'request', 0, $qry )) {
		my $pjn = db_get_projabb( $dbh, $pid );
		my $whr = "proj_id = $pid and status = 'act'";
		my $val = db_select_val( $dbh, 'metatext', 'maillst', $whr );
		my %msgvar = initmsg(
			mid	=> $memb_id,
			text	=> $val,
			proj	=> $pjn
		);
		$status = sendmsg( $dbh, 109, %msgvar );
	}
	return( $status );
}

=head2 confirm - confirmes a command

S<in-param:	$dbh - database handle
in-param:	$arg - argument (values)
returns:	$status - processing status>

=cut

sub confirm {
	my( $dbh, $arg ) = @_;
	debug( "confirm: arg: $arg", 1 );
	my $status = $r{nothing_confirm};
	$_ = $arg;
	/(.+)\.(\d+)\Z/;
	if( $1 && $2 ){
		my $qry = "cmdkey = '$1' and cmd_id = $2";
		if( db_select_val( $dbh, 'command', 'cmd_id', $qry )) {
			$status = $r{ok_confirmed};
			db_set_status( $dbh, $2, $status );
		}
	}
	return( $status );
}

=head2 filter - lists all personal mailfilters

S<in-param:	$dbh - database handle
in-param:	$mid - member id
returns:	$status - processing status>

=cut

sub filter {
	my( $dbh, $mid ) = @_;
	debug( "filter: mid: $mid", 1 );
	my $status = $r{error};
	my %msg = db_get_msg( $dbh, 10, 'de' );
	my %val = ();
	my $txt = '';
	my $i = 0;
	my $cur = db_cursor_open( $dbh,
		'member m, filt_memb f',			# tables
		'm.first_n, m.last_n, m.email',			# cols
		"m.memb_id = f.f_mid and f.memb_id = $mid"	# where
	);
	my $res = 1;
	while( $res ) {
		%val = db_cursor_get( $cur );
		$res = exists( $val{email} );
		if( $res ){
			if( $i == 0 ) { $txt .= "$msg{5}:\n"; }	# member
			$txt .= qq(- "$val{first_n} $val{last_n}" <$val{email}>\n);
			$i++;
		}
	}
	db_cursor_close( $cur );
	if( $i > 0 ) { $txt .= "\n"; }
	my $j = 0;
	$cur = db_cursor_open( $dbh,
		'metatext m, filt_proj f',			# tables
		'm.titlabb',					# cols
		"m.proj_id = f.f_pid and m.vers_id = 1 and f.memb_id = $mid"	# where
	);
	$res = 1;
	while( $res ) {
		%val = db_cursor_get( $cur );
		$res = exists( $val{titlabb} );
		if( $res ){
			if( $j == 0 ) { $txt .= "$msg{1}:\n"; }	# project
			$txt .= "- $val{titlabb}\n";
			$j++;
		}
	}
	db_cursor_close( $cur );
	if( $j > 0 ) { $txt .= "\n"; }
	$i += $j;
	$j = 0;
	$cur = db_cursor_open( $dbh, 'filt_mlst', 'f_mlst', "memb_id = $mid" );
	$res = 1;
	while( $res ) {
		%val = db_cursor_get( $cur );
		$res = exists( $val{f_mlst} );
		if( $res ){
			if( $j == 0 ) { $txt .= "$msg{2}:\n"; }	# mailinglist
			$txt .= "- $val{f_mlst}\n";
			$j++;
		}
	}
	db_cursor_close( $cur );
	$i += $j;
	my %msgvar = initmsg( mid => $mid, body => $txt, text => $i );
	$status = sendmsg( $dbh, 118, %msgvar );
	return( $status );
}

=head2 filtmemb - put a member into a personal mailfilter

S<in-param:	$dbh - database handle
in-param:	$mid - member id
in-param:	$arg - argument (member id to put in filter)
returns:	$status - processing status>

=cut

sub filtmemb {
	my( $dbh, $mid, $arg ) = @_;
	debug( "filtmemb: mid: $mid, arg: $arg", 1 );
	my $status = $r{error};
	my $whr = "memb_id = $mid and f_mid = $arg";
	if( db_select_val( $dbh, 'filt_memb', 'memb_id', $whr )) {
		if( db_delete( $dbh, 'filt_memb', $whr )) {
			$status = 115;		# exists: remove filter
		}
	} else {
		my %rec = ( memb_id => $mid, f_mid => $arg );
		if( db_insert( $dbh, 'filt_memb', %rec )) {
			$status = 112;		# don't exists: add filter
		}
	}
	if( $status > $r{ok} ){
		my $email = db_get_memb_name_email( $dbh, $arg );
		my %msgvar = initmsg( mid => $mid, text => $email );
		$status = sendmsg( $dbh, $status, %msgvar );
	}
	return( $status );
}

=head2 filtproj - put a project into a personal mailfilter

S<in-param:	$dbh - database handle
in-param:	$mid - member id
in-param:	$arg - argument (project id to put in filter)
returns:	$status - processing status>

=cut

sub filtproj {
	my( $dbh, $mid, $arg ) = @_;
	debug( "filtproj: mid: $mid, arg: $arg", 1 );
	my $status = $r{error};
	my $whr = "memb_id = $mid and f_pid = $arg";
	if( db_select_val( $dbh, 'filt_proj', 'memb_id', $whr )) {
		if( db_delete( $dbh, 'filt_proj', $whr )) {
			$status = 116;		# exists: remove filter
		}
	} else {
		my %rec = ( memb_id => $mid, f_pid => $arg );
		if( db_insert( $dbh, 'filt_proj', %rec )) {
			$status = 113;		# don't exists: add filter
		}
	}
	if( $status > $r{ok} ){
		my $email = db_get_projabb( $dbh, $arg );
		my %msgvar = initmsg( mid => $mid, text => $email );
		$status = sendmsg( $dbh, $status, %msgvar );
	}

	return( $status );
}

=head2 filtmlst - put a maillist into a personal mailfilter

S<in-param:	$dbh - database handle
in-param:	$mid - member id
in-param:	$arg - argument (maillist to put in filter)
returns:	$status - processing status>

=cut

sub filtmlst {
	my( $dbh, $mid, $arg ) = @_;
	debug( "filtmlst: mid: $mid, arg: $arg", 1 );
	my $status = $r{error};
	my $whr = "memb_id = $mid and f_mlst = '$arg'";
	if( db_select_val( $dbh, 'filt_mlst', 'memb_id', $whr )) {
		if( db_delete( $dbh, 'filt_mlst', $whr )) {
			$status = 117;		# exists: remove filter
		}
	} else {
		my %rec = ( memb_id => $mid, f_mlst => $arg );
		if( db_insert( $dbh, 'filt_mlst', %rec )) {
			$status = 114;		# don't exists: add filter
		}
	}
	if( $status > $r{ok} ){
		my %msgvar = initmsg( mid => $mid, text => $arg );
		$status = sendmsg( $dbh, $status, %msgvar );
	}

	return( $status );
}

=head2 mailfilter - filters address list with user filters

S<in-param:	$dbh - database handle
in-param:	$mid - member id to be fitered
in-param:	$pid - project id to be fitered
in-param:	$mlst - mailinglist to be filtered
in-param:	$addrlist - list of unfiltered mail addresses
returns:	$filtlist - list of filtered mail addresses>

=cut

sub mailfilter {
	my( $dbh, $mid, $pid, $mlst, @addrlist ) = @_;
	debug( "mailfilter: mid: $mid, pid: $pid, mlst: $mlst", 1 );
	my $status = $r{error};
	# first: determine mail addresses of user who set either
	# => a member filter (no mails from this member
	my @membfilt = db_select_col( $dbh,
		'member m, filt_memb f',			# tables
		'm.email',					# column
		"m.memb_id = f.memb_id and f.f_mid = $mid"	# where
	);
	my $curr_email = join( ', ', @membfilt );
	debug( "mailfilter.membfilt: $curr_email", 1 );
	# => or a project filter (no mails from this project)
	my @projfilt = db_select_col( $dbh,
		'member m, filt_proj f',			# tables
		'm.email',					# column
		"m.memb_id = f.memb_id and f.f_pid = $pid"	# where
	);
	$curr_email = join( ', ', @projfilt );
	debug( "mailfilter.projfilt: $curr_email", 1 );
	# => or a maillist filter (no mails from this maillist)
	my @mlstfilt = db_select_col( $dbh,
		'member m, filt_mlst f',			# tables
		'm.email',					# column
		"m.memb_id = f.memb_id and f.f_mlst = '$mlst'"	# where
	);
	$curr_email = join( ', ', @mlstfilt );
	debug( "mailfilter.mlstfilt: $curr_email", 1 );
	# second: unify filter list to one list with unique addr
	my %seen = ();
	my @filter = ();
	my $item;
	foreach $item (@membfilt) {
		push( @filter, $item ) unless $seen{$item}++;
	}
	foreach $item (@projfilt) {
		push( @filter, $item ) unless $seen{$item}++;
	}
	foreach $item (@mlstfilt) {
		push( @filter, $item ) unless $seen{$item}++;
	}
	$curr_email = join( ', ', @filter );
	debug( "mailfilter.unified_filter: $curr_email", 1 );
	# third: remove those addrs from addrlist which are in filter too
	my @filtlist = ();
	foreach $item (@addrlist) {
		push( @filtlist, $item ) unless exists $seen{$item};
	}
	$curr_email = join( ', ', @filtlist );
	debug( "mailfilter.resulting_filtered_list: $curr_email", 1 );
	return( @filtlist );
}

=head2 maillist - send an mail to mailinglist

S<in-param:	$dbh - database handle
in-param:	$rid - id of the receive record
in-param:	$pid - project id
in-param:	$arg - argument (values)
returns:	$status - processing status>

=cut

sub maillist {
	my( $dbh, $rid, $pid, $arg ) = @_;
	debug( "maillist: rid: $rid, pid: $pid, arg: arg", 1 );
	my $status = $r{error};
	if( $pid == 0 ) { return( $status ); };
	my $mlst = db_select_val( $dbh, 'metatext',
		'maillst',
		"proj_id = $pid and ( status = 'act' or status = 'fin' )"
	);
	debug( "maillist.mlst: $mlst", 1 );
	if( !( $mlst )) {
		mailerror( "ot_execute.maillist: empty mailinglist" );
		return( $r{error} );
	}
	my $whr = "m.memb_id = p.memb_id and p.proj_id = x.proj_id ";
	$whr .= "and x.maillst = '$mlst' and m.bounce < 5 and p.request = 0";
	my @mailaddr = db_select_col( $dbh,
		'member m, projmemb p, metatext x',		# tables
		'distinct m.email',				# column
		$whr						# where
	);
	my $curr_email = join( ', ', @mailaddr );
	debug( "maillist.curr_email: $#mailaddr: $curr_email", 1 );
	my %r = db_select_row( $dbh, 'receive',			# table
		'proj_addr, addr_to, addr_cc, subject, sender, send',	# fields
		"rec_id = $rid"					# where
	);
	my $long = $r{addr_to};
	if( $r{addr_cc} ) { $long .= "\nCc: ".$r{addr_cc}; }
	my $subj = $r{subject};
	$_ = $subj;
	if( !(/\[ot:$r{proj_addr}\]/ )) {
		$subj = "[ot:$r{proj_addr}] $subj";
	}
	my $from = $r{sender};
	my $cset = $r{send};
	my %msgvar = initmsg(
		from => $from,
		addr => $curr_email,
		long => $long,
		subj => $subj,
		cset => $cset,
		rply => "$mlst\@$ot{maildom}",
		body => $arg
	);
	$status = sendmsg( $dbh, 100, %msgvar );
	if( !is_err( $status )){
		if( !db_update_col( $dbh, 'metatext', 'mailnum', 'mailnum+1',
			"proj_id = $pid and status = 'act'" )){
			$status = $r{error};
		}
		my $t = "( $pid";
		$t = db_get_pidlist( $dbh, $pid, $t );
		$t .= ' )';
		my $pts = db_select_val( $dbh, 'global', 'pjpts' );
		$t = calc_activity( $dbh, $t, $pts );
 		if( !db_update_col( $dbh, 'metatext', 'actrate', $t,
			"proj_id = $pid and status = 'act'" )){
			$status = $r{error};
		}
	}
	return( $status );
}

=head2 setoption - sets an option for an ot member

S<in-param:	$dbh - database handle
in-param:	$mid - member id
in-param:	$arg - argument (values)
returns:	$status - processing status>

=cut

sub setoption {
	my( $dbh, $mid, $arg ) = @_;
	debug( "setoption: mid: $mid, arg: $arg", 1 );
	my $status = $r{error};
	my $v = 0;
	$_ = $arg;
	/(.+)=(.+)/;
	my $mask = 1 << $opt{$1};
	if( $2 eq 'n' ){ $mask = 65535 - $mask; }	# due to not having NOT-op in MySQL 23.22.x
	my $val = '(opt ';
	$val .= $2 eq 'y' ? "| $mask)" : "& $mask)";
	if( db_update_col( $dbh, 'member', 'opt', $val, "memb_id = $mid" )){
		$val = '';
		$mask = db_select_val( $dbh, 'member', 'opt', "memb_id = $mid" );
		my %tpo = reverse %opt;
		my %msg = db_get_msg( $dbh, 10, 'de' );
		foreach $v ( sort keys %tpo ){
			$val .= $tpo{$v}.' = ';
			$val .= ( 1 << $v ) & $mask ? "y ($msg{38})" : "n ($msg{39})";
			$val .= "\n";
		}
		my %msgvar = initmsg( mid => $mid, text => $arg, body => $val );
		$status = sendmsg( $dbh, 108, %msgvar );
	}
	return( $status );
}

=head2 sendopt - send a mail to members given by an option

S<in-param:	$dbh - database handle
in-param:	$arg - argument (first line: opt-pjn.passwd; rest: mailbody)
returns:	$status - processing status>

=cut

sub sendopt {
	my( $dbh, $rid, $arg ) = @_;
	debug( "send: arg: arg", 1 );
	my $status = $r{error};
	my( $tbl, $whr, $foot ) = ( '', '', 0 );
	@_ = split( /\n/, $arg );
	$_ = shift( @_ );		# remove first line
	debug( "send.erste_zeile: $_", 1 );
	$arg = join( "\n", @_ );
	/(.+?)-(.+)\..+=(\d+)\.(\d+)\.(.+)/;
	my( $op, $pjn, $mid, $pid, $mlst ) = ( $1, $2, $3, $4, $5 );
	debug( "send: op: $op, pjn: $pjn, mid: $mid, pid: $pid, mlst: $mlst", 1 );
	my $i = $opt{$op};
	my $mask1 = 1 << $i;
	my $mask2 = $mask1;
	$_ = $op;
	debug( "send.option: $op" , 2 );
	if( /newsletter/ ){
		$tbl = 'member m';
	}
	elsif( /newsmaintainer/ ){
		$tbl = 'member m, projmemb p';
		$whr = "m.memb_id = p.memb_id and p.ismaint = 1 and ";
	}
	elsif( /newproject/ ){
		$tbl = 'member m';
		$foot = 1;
	}
	elsif( /newversionall/ ){
		$tbl = 'member m';
		$foot = 1;
	}
	elsif( /newversionmy/ ){
		$mask1 += 1 << $opt{newversionall};	# only having not ...all
#		my $pid = db_is_project( $dbh, $pjn );
		$tbl = 'member m, projmemb p';
		$whr = "m.memb_id = p.memb_id and p.proj_id = $pid and ";
		$foot = 1;
	}
	else {
		return( $status );
	}
	$whr .= "( m.opt & $mask1 ) = $mask2 and m.bounce < 5";
	my @mailaddr = db_select_col( $dbh, $tbl, 'distinct m.email', $whr );
	my $curr_email = join( ', ', @mailaddr );
	debug( "send.curr_email: $#mailaddr: $curr_email", 1 );
	if( $#mailaddr < 0 ){ return( $r{no_recipients} ); }
	@mailaddr = mailfilter( $dbh, $mid, $pid, $mlst, @mailaddr );
	$curr_email = join( ', ', @mailaddr );
	debug( "send.filt_email: $#mailaddr: $curr_email", 1 );
	my %r = db_select_row( $dbh, 'receive',		# table
		'subject, sender',			# fields
		"rec_id = $rid"				# where
	);
	if( $foot == 1 ){				# add footer
		my $membfilt = "filtmemb*$mid\@$ot{maildom}";
		my $projfilt = "filtproj*$pid\@$ot{maildom}";
		my $mlstfilt = "filtmlst*$mlst\@$ot{maildom}";
		if( !( $_ = db_get_mail_msg( $dbh, 800, 'de' ))) {
			mailerror( "ot_execute.sendopt: missing msg" );
			return( $r{error} );
		}
		$arg .= eval qq( "$_" );		# interpret in context!
	}
	my $long = "$op-list <anonym@".$ot{maildom}.">";
	my $rply = db_select_val( $dbh, 'member', 'email', "memb_id = $mid" );
	my %msgvar = initmsg(
		from => $r{sender},
		addr => $curr_email,
		long => $long,
		subj => "[ot-$op] $r{subject}",
		rply => $rply,
		body => $arg
	);
	$status = sendmsg( $dbh, 200+$i, %msgvar );
	return( $status );
}

=head2 eval_command - evaluates and executes a command

S<in-param:	$dbh - database handle
in-param:	$rid - id of the receive record
in-param:	$cid - id of the command record
in-param:	$pid - project id
in-param:	$mid - member Id (0, if not a ot-member)
in-param:	$cmd - command
in-param:	$arg - argument (values)
returns:	$status - processing status>

=cut

sub eval_command {
	my( $dbh, $rid, $cid, $pid, $mid, $cmd, $arg ) = @_;
	my $status = $r{error};
	my %rec;
	debug( "eval_command: rid: $rid, cid: $cid, pid: $pid, mid: $mid, cmd: $cmd, arg: arg", 1 );

	# commands without arguments:
	# which, who, subscribe, unsubscribe, gettree, getlists
	if( is( $cmd, $f{no_arg} )) {
		$_ = $cmd;
		if( /which/ ){
			$status = which( $dbh, $mid );
		}
		elsif( /who/ ){
			$status = who( $dbh, $pid, $mid );
		}
		elsif( /unsubscribe/ ){
			$status = unsubscribe( $dbh, $pid, $mid );
		}
		elsif( /subscribe/ ){
			$status = subscribe( $dbh, $pid, $mid, $arg );
		}
		elsif( /filter/ ) {
			$status = filter( $dbh, $mid );
		}
		elsif( /getlists/ ){
		}
		elsif( /gettree/ ){
		}
	}

	# commands with one argument:
	# filt*, commit, confirm, help, submit, give, addsub, uselist, keywords, title, description
	elsif( is( $cmd, $f{one_arg} )) {
		$_ = $cmd;
		if( /addsub/ ){
		}
		elsif( /commit/ ){
			$status = commit( $dbh, $pid, $mid, $arg );
		}
		elsif( /confirm/ ){
			$status = confirm( $dbh, $arg );
		}
		elsif( /description/ ){
		}
		elsif( /filtmemb/ ){
			$status = filtmemb( $dbh, $mid, $arg );
		}
		elsif( /filtproj/ ){
			$status = filtproj( $dbh, $mid, $arg );
		}
		elsif( /filtmlst/ ){
			$status = filtmlst( $dbh, $mid, $arg );
		}
		elsif( /give/ ){
		}
		elsif( /help/ ){
		}
		elsif( /keywords/ ){
		}
		elsif( /submit/ ){
		}
		elsif( /title/ ){
		}
		elsif( /uselist/ ){
		}
	}

	# commands with two arguments:
	# setoption, comment, setname, newversion
	elsif( is( $cmd, $f{two_args} )) {
		$_ = $cmd;
		if( /setoption/ ){
			$status = setoption( $dbh, $mid, $arg );
		}
		elsif( /comment/ ){
		}
		elsif( /setname/ ){
		}
		elsif( /newversion/ ){
		}
		elsif( /maillist/ ){
			$status = maillist( $dbh, $rid, $pid, $arg );
		}
	}

	# commands with three arguments:
	# newproject
	elsif( is( $cmd, $f{three_args} )) {
		$_ = $cmd;
		if( /newproject/ ){
		}
		elsif( /send/ ){
			$status = sendopt( $dbh, $rid, $arg );	# two args had only checking purposes
		}
	}

	if( $status == $r{ok_confirmed} ){	# if a cmd was confirmed: execute cmd
		$status = $r{nothing_confirm};
		$_ = $arg;
		/.+\.(\d+)\Z/;
		if( $1 ) {
			%rec = db_select_row( $dbh,
			'receive r, command c',					# tables
			'r.rec_id, r.proj_id, r.memb_id, c.cmdkey, c.cmdval',	# fields
			"r.rec_id = c.rec_id and c.cmd_id = $1
			and c.stcode = $r{ok_confirmed}"			# where
			);
			if( exists( $rec{proj_id} )) {
				$status = eval_command( $dbh, $rec{rec_id}, $1, $rec{proj_id},
					$rec{memb_id}, $rec{cmdkey}, $rec{cmdval} );
			}
		}
	}

	db_set_status( $dbh, $cid, $status );
	return( $status );
}

=head2 send_confirm - sends a mail to user to confirm a command

S<in-param:	$dbh - database handle
in-param:	$cid - id of the command record
in-param:	$long - sender's long address
in_param:	$subj - subject of user mail
in-param:	$cmd - command
returns:	$status - processing status>

=cut

sub send_confirm {
	my( $dbh, $cid, $long, $subj, $cmd ) = @_;
	my $status = $r{ok_requested};
	debug( "send_confirm: cid: $cid, long: $long, subj: $subj, cmd: $cmd", 1 );
	my %msgvar = initmsg(
		long => $long,
		subj => $subj,
		rply => "confirm*$cmd.$cid\@$ot{maildom}",
		cmd => $cmd
	);
	sendmsg( $dbh, 101, %msgvar );
	db_set_status( $dbh, $cid, $status );
	return( $status );
}

=head2 eval_error - evaluates an error and sends an user reply

S<in-param:	$dbh - database handle
in-param:	$cid - id of the command record
in-param:	$long - sender's long address
in_param:	$subj - subject of user mail
in-param:	$cmd - command
in-param:	$body - mail body of user mail
in-param:	$stc - status code of prior evaluation
returns:	$status - processing status>

=cut

sub eval_error {
	my( $dbh, $cid, $pid, $long, $subj, $cmd, $val, $body, $stc ) = @_;
	my $status = $r{ok_finished};
	my $pjn = '';
	if( $pid ) { $pjn = db_get_projabb( $dbh, $pid ); }
	debug( "eval_error: long: $long, subj: $subj, stc: $stc", 1 );
	my %msgvar = initmsg(
		long => $long,
		subj => $subj,
		cmd => $cmd,
		proj => $pjn,
		text => $val,
		body => $body
	);
	if( $stc <= $r{spam_suspicion} ){
		debug( "+++++ spam suspicion: no reply sent to user +++++" );
	} else {
		sendmsg( $dbh, $stc, %msgvar );
	}
	db_set_status( $dbh, $cid, $status );
	return( $status );
}

=head2 execute - executes all commands

C<in-param:	$msg_id - message id
returns:	$status>

=cut

sub execute {
	debug( "##### Start execute: ".localtime, 0 );
	my( $msg_id ) = @_;
	my $status = $r{ok};
	my $dbh = db_connect;	# database handle
	my @rlist = db_select_col( $dbh, 'receive', 'rec_id', "msg_id = '$msg_id'" );
	my @clist = ();
	my @t = ();
	my $rec_id;
	my $cmd_id;
	my %x;

	foreach $rec_id ( @rlist ) {
		@t = db_select_col( $dbh, 'command', 'cmd_id', "rec_id = $rec_id" );
		push( @clist, @t );
	}

	# first: send confirm
	foreach $cmd_id ( @clist ){
		%x = db_select_row( $dbh,
			'receive r, command c',						# tables
			'r.sender, r.subject, c.cmdkey',				# fields
			"r.rec_id = c.rec_id and c.cmd_id = $cmd_id
			and c.stcode = $r{ok_to_confirm}"				# where
		);
		if( exists( $x{sender} )) {
			$status = send_confirm( $dbh, $cmd_id, $x{sender},
				$x{subject}, $x{cmdkey} );
		}
	}

	# second: evaluate all commands
	foreach $cmd_id ( @clist ){
		%x = db_select_row( $dbh,
			'receive r, command c',						# tables
			'r.rec_id, r.proj_id, r.memb_id, c.cmdkey, c.cmdval',		# fields
			"r.rec_id = c.rec_id and c.cmd_id = $cmd_id
			and c.stcode >= $r{ok} and c.stcode < $r{ok_finished}"		# where
		);
		if( exists( $x{proj_id} )) {
			$status = eval_command( $dbh, $x{rec_id}, $cmd_id,
				$x{proj_id}, $x{memb_id}, $x{cmdkey}, $x{cmdval} );
		}
	}

	# third: evaluate all execute errors
	foreach $cmd_id ( @clist ){
		%x = db_select_row( $dbh,
			'receive r, command c',						# tables
			'r.proj_id, r.sender, r.subject, r.body, c.stcode, c.cmdkey, c.cmdval',	# fields
			"r.rec_id = c.rec_id and c.cmd_id = $cmd_id
			and c.stcode < $r{errors}"					# where
		);
		if( exists( $x{sender} )) {
			$status = eval_error( $dbh, $cmd_id, $x{proj_id}, $x{sender},
				$x{subject}, $x{cmdkey}, $x{cmdval}, $x{body}, $x{stcode} );
		}
	}

	# fourth: delete all finished commands
	my $r = 0;
	@rlist = db_select_col( $dbh, 'command', 'rec_id', "stcode = $r{ok_finished}" );
	if( !db_delete( $dbh, 'command', "stcode = $r{ok_finished}" )) {last;}
	foreach $rec_id ( sort @rlist ) {
		if( $r != $rec_id ) {
			@t = db_select_col( $dbh, 'command', 'cmd_id', "rec_id = $rec_id" );
			if( $#t < 0 ) {
				if( !db_delete( $dbh, 'receive', "rec_id = $rec_id" )) {last;}
			}
		}
		$r = $rec_id;
	}

	db_disconnect( $dbh );
	return( $status );
}
1;
